function visual_features = read_visual_features()
%% Visual features for the MOCHA database
%
% as provided by the 
% Computer Vision, Speech Communication & Signal Processing Group 
% at the National Technical University of Athens, Greece
%
% CVSP group web site:
% http://cvsp.cs.ntua.gr
%
%% Overview:
% --------------------
% The MOCHA (Multichannel Articulatory) database has been compiled by the
% Department of Speech and Language Sciences at Queen Margaret University
% College and the Department of Linguistics at the University of Edinburgh.
% It can be found online at
% http://www.cstr.ed.ac.uk/research/projects/artic/mocha.html
%
% The database currently comprises sound, EMA (Electromagnetic
% Articulography), laryngograph, electropalatograph and video recordings of
% two speakers (one male and one female) uttering a set of 460 sentences in
% English. For details on the database please refer to (Wrench, Hardcastle
% 2000). Though the rest of the recorded modalities have been exploited in
% various contexts (Richmond et al. 2003, Toda et al. 2008, Toutios and
% Margaritis 2008) the video footage had been so far unused.
%
% To also use the video for audiovisual speech inversion
% (Katsamanis et al. 2008, 2009) we processed the raw recordings of the
% female speaker (fsew0) and extracted visual features based on face active
% appearance modeling (Cootes et al. 2001). The video was first segmented
% and labeled by automatically aligning the pretranscribed audio data
% (available with the MOCHA database) with audio tracks extracted from the
% video files. Shape and texture features were extracted at 25Hz, for each
% video frame, after face detection, tracking and active appearance model
% fitting, using the AAM fitting algorithm described in (Papandreou and
% Maragos, 2008). Please check the AAMtools webpage
% (http://cvsp.cs.ntua.gr/software/AAMtools) for further information
% related to the modeling process and software. In total, 12 shape and 27
% texture features were extracted from each frame. Synchronization with the
% EMA features was verified both visually and quantitatively using
% canonical correlation analysis (Katsamanis et al. 2009).
% 
%% Practical details:
% -------------------------------
% Features are stored in binary HTK format (Young et al. 2000) in a
% separate file per utterance. Naming of the files follows the MOCHA
% conventions. The suffix is .aam. 
%
% The current script demonstrates how to import the MOCHA visual features
% in the MATLAB environment.
%
% Please check http://cvsp.cs.ntua.gr/research/inversion for further
% description of our related research and updates on papers, datasets and
% software.
% 
%% References:
% -----------------------
% A. Wrench and W. Hardcastle, "A multichannel articulatory speech database
%   and its application for automatic speech recognition," in Proc. 5th
%   Seminar on Speech Production, Kloster Seeon, Bavaria, 2000, pp. 305-308.
%   [Online]. Available: http://www.cstr.ed.ac.uk/artic  
% K. Richmond, S. King, and P. Taylor, "Modelling the uncertainty in
%   recovering articulation from acoustics," Computer Speech and Language,
%   vol. 17, pp. 153-172, 2003. 
% T. Toda, A. W. Black, and K. Tokuda, "Statistical mapping between
%   articulatory movements and acoustic spectrum using a gaussian mixture
%   model," Speech Communication, vol. 50, pp. 215-227, 2008. 
% A. Toutios, K. Margaritis, "Estimating Electropalatographic Patterns from
%   the Speech Signal", Computer Speech and Language, Volume 22, Issue 4,
%   Pages 346-359, October 2008
% A. Katsamanis, G. Papandreou, and P. Maragos, "Face Active Appearance
%   Modeling and Speech Acoustic Information to Recover Articulation," IEEE
%   Transactions on Audio, Speech and Language Processing, to appear.
% A. Katsamanis, G. Papandreou, and P. Maragos, "Audiovisual-to-Articulatory
%   Speech Inversion Using Active Appearance Models for the Face and Hidden
%   Markov Models for the Dynamics," Proc. IEEE Int'l Conference on Acoustics,
%   Speech, and Signal Processing (ICASSP-2008), Las Vegas, NV, U.S.A.,
%   Mar.-Apr. 2008.
% T. Cootes, G. Edwards, and C. Taylor, "Active Apperance Models," IEEE
%   Transactions on Pattern Analysis and Machine Intelligence, Vol.23,
%   No.6, pp. 681-685, June 2001
% G. Papandreou and P. Maragos, "Adaptive and Constrained Algorithms for
%   Inverse Compositional Active Appearance Model Fitting", Proc. IEEE
%   Int'l Conf. on Computer Vision and Pattern Recognition (CVPR-2008),
%   Anchorage, AL, June 2008.
% S. Young et al., The HTK Book (for HTK Version 3.0), University of
%   Cambridge, 2000. [Online]. Available: http://htk.eng.cam.ac.uk/docs/docs.shtml 
%
% For reprints of our papers, visit the CVSP group web site
% URL: http://cvsp.cs.ntua.gr
%
%%

plot_shape=true;
plot_appearance=true;
animate_shape=true;

vfeat_id='aam_mouth-roi_fsew0_nofit_shape12texture27_25hz';
visual_model = fullfile('fsew0','visual',vfeat_id,'model','aamModel.mat');
visual_path = fullfile('fsew0','visual',vfeat_id,'htk');
visual_suffix = 'aam';
fps = 25;
n_shape_features = 12;
n_texture_features = 27;
normalize_mean = true;

visual_files = find_files_in_dir(visual_path, visual_suffix);
n_visual_files = length(visual_files);

% Read the visual features for each utterance
visual_features = cell(n_visual_files,1);
for k=1:n_visual_files
    k_visual_file = visual_files{k};
    k_visual_file = fullfile(visual_path, k_visual_file);    
    [k_header, k_data] = readHTK(k_visual_file);
    if normalize_mean
        k_data = k_data - repmat(mean(k_data,2),1,size(k_data,2));
    end
    visual_features{k} = k_data;
end

%% Plot all visual features
n_rows = 3;
feat_mat = cell2mat(visual_features');
if plot_shape,
  % Shape features
  h_shape = figure;
  set(h_shape,'Name', 'Shape Features');
  n_columns = n_shape_features/n_rows;
  for k=1:n_shape_features
    subplot(n_rows, n_columns, k);
    plot(feat_mat(k,:));
  end
end
if plot_appearance,
  % Appearance features
  h_appearance = figure;
  set(h_appearance,'Name', 'Appearance Features');
  n_columns = n_texture_features/n_rows;
  for k=1:n_texture_features
    subplot(n_rows, n_columns, k);
    plot(feat_mat(n_shape_features+k,:));
  end
end

%% Animate Shape
if animate_shape,
  tmp=load(visual_model);
  aamModel = tmp.aamModel;
  h_anim = figure;
  % Select which utterances to animate
  %for k=1:n_visual_files
  for k=1:50:n_visual_files
    shape_feat_k = visual_features{k}(1:n_shape_features,:);
    figure(h_anim);
    [tmp,utter_id] = fileparts(visual_files{k});
    utter_id = strrep(utter_id,'_','\_');
    for t=1:size(shape_feat_k,2)
      tic,
      shape = synthesizeShape(aamModel.shapes,shape_feat_k(:,t));
      scatterPlot2DShape(shape);
      title(utter_id);
      drawnow,
      pause(1/fps-toc);
    end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function file_cell = find_files_in_dir(directory, suffix)
% FIND_FILES_IN_DIR - Find files of a specific type in a given directory
%
% file_cell = find_files_in_dir(directory, suffix);
%
% Description:
% Search in a directory recursively for files with the given suffix. The
% filenames are returned. Their path is relative to the given directory.
%
% Input arguments:
% directory: The directory to search in 
%
% Output arguments:
% file_cell: Cell of the found filenames
% 
% Example:
% % Look for wav files in the current directory
% fc = find_files_in_dir(pwd, 'wav);
% 
% Project: File Utilities
%
dir_contents = dir(directory);

file_cell = {};
for k=1:length(dir_contents)
    dir_entry = dir_contents(k);
    if dir_entry.isdir
        if isempty(regexp(dir_entry.name,'^\.'))
            f_cell = find_files_in_dir(fullfile(directory,dir_entry.name),suffix);
            for l=1:length(f_cell)
                file_cell{end+1} = fullfile(dir_entry.name,f_cell{l});
            end
        else
            continue;
        end
    else
        [pth,bsnm,sfx] = fileparts(dir_entry.name);
        if ~isempty(findstr(suffix,'.'))
            [t,r] = strtok(bsnm,'.');
            sfx = [r, sfx];
        end
        if strmatch(sfx,['.',suffix],'exact')
            file_cell{end+1} = dir_entry.name;
        end
    end
end

function [header, data] = readHTK(inputFileName)
% READHTK - Read the header and contents of a file in HTK format
% 
%   [header, data] = readHTK(inputFileName)
%   
% Description:
% Follow the guidelines in HTKBook and read the header of an
% HTK-formatted file. Data are stored as 4-byte floats and a 12-byte header
% is prepended.
%
% Input Arguments:
% inputFileName: The name of the file in HTK format.
% 
% Output Arguments:
% header: The header of the file, a 5 field struct including the number of samples,
%             and the sampling period in 100nsec.
% data: A NDIMxNFRAMES matrix. NDIM is the dimension of the dataset while
%             NFRAMES is the number of data frames.
% 
% Example: 
% 
% Project: HTK Tools
%   
fid = fopen(inputFileName);
if fid==-1 
  error(['Cannot open ', inputFileName]);
end

nSamples = fread(fid, 1, 'int32');
sampPeriod = fread(fid, 1, 'int32');
sampSize = fread(fid, 1, 'int16');
parmKind = fread(fid, 1, 'int8');
qualifier = fread(fid, 1, 'int8');

header.nSamples = nSamples;
header.sPeriod = sampPeriod;
header.sampSize = sampSize;
header.parmKind = parmKind;
header.qualifier = qualifier;

tmp = fread(fid, Inf, 'float32');
data = reshape(tmp, sampSize/4, nSamples);
fclose(fid);


function shape = synthesizeShape(shapes,p)
% SYNTHESIZESHAPE - Synthesize shape from the AAM shape parameters
%
% shape = synthesizeShape(shapes,p);
%
% Description:
% Given a mean shape shapes(:,:,1) and eigenshapes shapes(:,:,2:end),
% and the shape coords p - corresponding to deformations W_p
% compute the deformed shape = meanShape + \sum p_i*eigenShape(i)
%
% Input arguments:
% shapes: mean shape shapes(:,:,1) and eigenshapes shapes(:,:,2:end)
% p     : shape expansion coordinates in the eigenshape basis
%
% Output arguments:
% shape : synthesized shape
% 
% Project: AAMtools
% http://cvsp.cs.ntua.gr/software/AAMtools
%

if nargin <2, error('Not enough args provided'); end

numEigenShapes = length(p);
if size(shapes,3)-1 ~= numEigenShapes, error('incompatible dims'); end

[L num_dims num_shapes]=size(shapes);
shapes=reshape(shapes, [L*num_dims num_shapes]);
p_1=[1; p(:)];
shape = shapes*p_1;
shape=reshape(shape, [L num_dims]);


function scatterPlot2DShape(coords)
% SCATTERPLOT2DSHAPE - Plot shape
%
% scatterPlot2DShape(coords);
%
% Description:
% Plots the shape in coords and its Delaunay triangulation
%
% Input arguments:
% coords: shape coordinates, x-coords in coords(:,1) and y-coords in coords(:,2) 
%
% Project: AAMtools
% http://cvsp.cs.ntua.gr/software/AAMtools
%

if nargin<1, error('You did not provide the point coords'); end

plot_args = {'LineStyle','none','LineWidth',1,'Color','b',...
  'Marker','x','MarkerSize',5,'MarkerEdgeColor','b'};
plot(coords(:,1),coords(:,2),plot_args{:});
axis ij, axis equal; axis off;

holdFlag = ishold; hold on;
tri = delaunay(coords(:,1),coords(:,2));
% Plot the Delaunay Triangulation
delaunay_args = {'LineStyle','-', 'LineWidth',1,'Color','b'};
triplot(tri,coords(:,1),coords(:,2),delaunay_args{:});
if ~holdFlag, hold off; end
